
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef EKGFREQ_H
#define EKGFREQ_H

#include <sys/time.h>
#include <cmath>
#include <iostream>
#include <string>
#include "EKGdevice.h"
#include "calibrate.h"
using namespace std;

/**
 * Diese Klasse implementiert einen Pulsfrequenzzhler.  Die Puslfrequenz
 * kann auf stderr ausgegeben werden (setFreq()).  Auerdem kann  bei jedem
 * Herzschlag ein Piep (^G) auf stderr ausgegeben werden (setBeep()).
 *
 * @author Lars Stollenwerk, Oktober 2002
 */
class EKGfreq : public EKGdevice
{
  public:
    /**
     * Konstruktor.
     */
    EKGfreq() : beepFlag(false), freqFlag(false), freq(0), lastQRS(0),
                Pthreshold(0), Nthreshold(0), Zthreshold(0), QRS(false) 
		{ setDecay(1.0); }
    /**
     * Piepsen auf cerr ein/aus schalten. 
     * @param x true = ein
     * @return bisheriger Zustand
     */
    bool setBeep( bool x );
    /** 
     * Pulsfrequenz-Ausgabe auf cerr ein/aus schalten.
     * @param x true = ein
     * @return bisheriger Zustand
     */
    bool setFreq( bool x );
    /**
     * Setzt Zeitkonstante fr Sinken der Schwelle in Sekunden.
     * @param x Neue Zeitkonstante in Sekunden.
     * @return bisherige Zeitkonstante in Sekunden.
     */
    float setDecay( float x );
    /**
     * berschreiben von EKGdevice::input(). Hier findet die
     * Datenverarbeitung statt.
     */
    virtual void input( float * d );
  protected:
    /**
     * Flag fr piepsen.
     */
    bool beepFlag;
    /**
     * Flag fr Frequenzausgabe.
     */
    bool freqFlag;
    /**
     * gemessene Pulsfrequenz.
     */
    int freq;
    /**
     * Zeitpunkt des letzten QRS-Komplex. Diese Angabe wird von calcFreq()
     * gebraucht.
     */
    double lastQRS;
    /**
     * Auf QRS-Komplex reagieren.
     */
    virtual void handleQRS();
    /**
     * Piepsen, falls eingeschaltet. Gibt ^G auf stderr aus.
     * Wird von handleQRS aufgerufen.
     */
    void beep(){ if ( beepFlag ) cerr << '\a'; }
    /**
     * Gibt Pusfrequenz aus, falls eingeschaltet.
     * Wird von handleQRS aufgerufen.
     */
    void printFreq(){ if ( freqFlag ) cerr << "Pulse : " << freq << "     \r"; }
    /**
     * Mit Pulsfrequenz. Die Pulsfrequenz wird anhand zweier
     * aufeinanderfolgender QRS-Komplexe gemessen.
     * Wird von handleQRS aufgerufen.
     */
    void calcFreq();
    // ----------------------------------------------------------------------
    // QRS-Komplex Detektor
    // ----------------------------------------------------------------------
    /**
     * Positive Schwelle fr QRS-Komplex.
     */
    float Pthreshold;
    /**
     * Negative Schwelle fr QRS-Komplex.
     */
    float Nthreshold;
    /**
     * Mittelwert, der als Bezugspunkt fr die Schwellen dient.
     * Wird aus Mittelung mit 10-facher decay-Zeit gebildet.
     */
    float Zthreshold;
    /**
     * Zerfallskonstante fr Sinken der Schwelle. Die Schwelle wird nach
     * jedem Wertetripel mit decay multipliziert. Es mu also decay < 1
     * sein.
     */
    float decay;
    /** 
     * Zerfallskonstante fr Mittelwert Zthreshold.
     * Die Zerfallszeit fr den Mittlewert ist zehn mal so lang wie die fr
     * die Schwellwerte, also ist zdecay = decay / 10.
     */
    float zdecay;
    /**
     * Jetzt ist QRS Flag. Ist true whrend eines QRS-Komplex.
     */
    bool QRS;
    /**
     * Prft berschrteiten der positiven oder unterschreiten der negativen
     * Schwelle. Es wir nur mit der betragsmig greren Schwelle
     * verglichen.
     * @param x zu berprfender Wert
     * @return ber/unterschreiten einer Schwelle
     */
    bool aboveThreshold( const float x );
};

#endif
